@@ -1,11 +1,13 @@
-from dataclasses import dataclass, field, fields
+from dataclasses import asdict, dataclass, field, fields
 from functools import cached_property
 from json import loads
+from logging import warning
 from os import PathLike
+from platform import system
 from re import split
 from subprocess import CompletedProcess
 from subprocess import run as subprocess_run
-from typing import Callable, Literal, Optional, TypeVar, Union
+from typing import Any, Callable, Literal, Optional, TypeVar, Union, cast
 from urllib.error import HTTPError, URLError
 from urllib.request import urlopen
 
@@ -13,6 +15,7 @@ from testcontainers.core.exceptions import ContainerIsNotRunning, NoSuchPortExpo
 from testcontainers.core.waiting_utils import wait_container_is_ready
 
 _IPT = TypeVar("_IPT")
+_WARNINGS = {"DOCKER_COMPOSE_GET_CONFIG": "get_config is experimental, see testcontainers/testcontainers-python#669"}
 
 
 def _ignore_properties(cls: type[_IPT], dict_: any) -> _IPT:
@@ -38,6 +41,14 @@ class PublishedPort:
     PublishedPort: Optional[str] = None
     Protocol: Optional[str] = None
 
+    def normalize(self):
+        url_not_usable = system() == "Windows" and self.URL == "0.0.0.0"
+        if url_not_usable:
+            self_dict = asdict(self)
+            self_dict.update({"URL": "127.0.0.1"})
+            return PublishedPort(**self_dict)
+        return self
+
 
 OT = TypeVar("OT")
 
@@ -156,9 +167,11 @@ class DockerCompose:
     pull: bool = False
     build: bool = False
     wait: bool = True
+    keep_volumes: bool = False
     env_file: Optional[str] = None
     services: Optional[list[str]] = None
     docker_command_path: Optional[str] = None
+    profiles: Optional[list[str]] = None
 
     def __post_init__(self):
         if isinstance(self.compose_file_name, str):
@@ -169,7 +182,7 @@ class DockerCompose:
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb) -> None:
-        self.stop()
+        self.stop(not self.keep_volumes)
 
     def docker_compose_command(self) -> list[str]:
         """
@@ -186,6 +199,8 @@ class DockerCompose:
         if self.compose_file_name:
             for file in self.compose_file_name:
                 docker_compose_cmd += ["-f", file]
+        if self.profiles:
+            docker_compose_cmd += [item for profile in self.profiles for item in ["--profile", profile]]
         if self.env_file:
             docker_compose_cmd += ["--env-file", self.env_file]
         return docker_compose_cmd
@@ -227,6 +242,10 @@ class DockerCompose:
             down_cmd += ["down", "--volumes"]
         else:
             down_cmd += ["stop"]
+
+        if self.services:
+            down_cmd.extend(self.services)
+
         self._run_command(cmd=down_cmd)
 
     def get_logs(self, *services: str) -> tuple[str, str]:
@@ -244,6 +263,36 @@ class DockerCompose:
         result = self._run_command(cmd=logs_cmd)
         return result.stdout.decode("utf-8"), result.stderr.decode("utf-8")
 
+    def get_config(
+        self, *, path_resolution: bool = True, normalize: bool = True, interpolate: bool = True
+    ) -> dict[str, Any]:
+        """
+        Parse, resolve and returns compose file via `docker config --format json`.
+        In case of multiple compose files, the returned value will be a merge of all files.
+
+        See: https://docs.docker.com/reference/cli/docker/compose/config/ for more details
+
+        :param path_resolution: whether to resolve file paths
+        :param normalize: whether to normalize compose model
+        :param interpolate: whether to interpolate environment variables
+
+        Returns:
+            Compose file
+
+        """
+        if "DOCKER_COMPOSE_GET_CONFIG" in _WARNINGS:
+            warning(_WARNINGS.pop("DOCKER_COMPOSE_GET_CONFIG"))
+        config_cmd = [*self.compose_command_property, "config", "--format", "json"]
+        if not path_resolution:
+            config_cmd.append("--no-path-resolution")
+        if not normalize:
+            config_cmd.append("--no-normalize")
+        if not interpolate:
+            config_cmd.append("--no-interpolate")
+
+        cmd_output = self._run_command(cmd=config_cmd).stdout
+        return cast(dict[str, Any], loads(cmd_output))
+
     def get_containers(self, include_all=False) -> list[ComposeContainer]:
         """
         Fetch information about running containers via `docker compose ps --format json`.
@@ -357,7 +406,7 @@ class DockerCompose:
         str:
             The mapped port on the host
         """
-        return self.get_container(service_name).get_publisher(by_port=port).PublishedPort
+        return self.get_container(service_name).get_publisher(by_port=port).normalize().PublishedPort
 
     def get_service_host(
         self,
@@ -379,14 +428,14 @@ class DockerCompose:
         str:
             The hostname for the service
         """
-        return self.get_container(service_name).get_publisher(by_port=port).URL
+        return self.get_container(service_name).get_publisher(by_port=port).normalize().URL
 
     def get_service_host_and_port(
         self,
         service_name: Optional[str] = None,
         port: Optional[int] = None,
     ):
-        publisher = self.get_container(service_name).get_publisher(by_port=port)
+        publisher = self.get_container(service_name).get_publisher(by_port=port).normalize()
         return publisher.URL, publisher.PublishedPort
 
     @wait_container_is_ready(HTTPError, URLError)
