@@ -10,25 +10,48 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import contextlib
 import functools as ft
 import importlib.metadata
 import ipaddress
 import os
+import socket
 import urllib
 import urllib.parse
-from os.path import exists
-from pathlib import Path
-from typing import Optional, Union
+from collections.abc import Iterable
+from typing import Callable, Optional, TypeVar, Union
 
 import docker
 from docker.models.containers import Container, ContainerCollection
+from docker.models.images import Image, ImageCollection
+from typing_extensions import ParamSpec
 
+from testcontainers.core import utils
+from testcontainers.core.auth import DockerAuthInfo, parse_docker_auth_config
+from testcontainers.core.config import ConnectionMode
+from testcontainers.core.config import testcontainers_config as c
 from testcontainers.core.labels import SESSION_ID, create_labels
-from testcontainers.core.utils import default_gateway_ip, inside_container, setup_logger
 
-LOGGER = setup_logger(__name__)
-TC_FILE = ".testcontainers.properties"
-TC_GLOBAL = Path.home() / TC_FILE
+LOGGER = utils.setup_logger(__name__)
+
+_P = ParamSpec("_P")
+_T = TypeVar("_T")
+
+
+def _wrapped_container_collection(function: Callable[_P, _T]) -> Callable[_P, _T]:
+    @ft.wraps(ContainerCollection.run)
+    def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> _T:
+        return function(*args, **kwargs)
+
+    return wrapper
+
+
+def _wrapped_image_collection(function: Callable[_P, _T]) -> Callable[_P, _T]:
+    @ft.wraps(ImageCollection.build)
+    def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> _T:
+        return function(*args, **kwargs)
+
+    return wrapper
 
 
 class DockerClient:
@@ -41,13 +64,20 @@ class DockerClient:
 
         if docker_host:
             LOGGER.info(f"using host {docker_host}")
-            self.client = docker.DockerClient(base_url=docker_host)
+            os.environ["DOCKER_HOST"] = docker_host
+            self.client = docker.from_env(**kwargs)
         else:
             self.client = docker.from_env(**kwargs)
         self.client.api.headers["x-tc-sid"] = SESSION_ID
         self.client.api.headers["User-Agent"] = "tc-python/" + importlib.metadata.version("testcontainers")
 
-    @ft.wraps(ContainerCollection.run)
+        # Verify if we have a docker auth config and login if we do
+        if docker_auth_config := get_docker_auth_config():
+            LOGGER.debug(f"DOCKER_AUTH_CONFIG found: {docker_auth_config}")
+            if auth_config := parse_docker_auth_config(docker_auth_config):
+                self.login(auth_config[0])  # Only using the first auth config)
+
+    @_wrapped_container_collection
     def run(
         self,
         image: str,
@@ -81,6 +111,17 @@ class DockerClient:
         )
         return container
 
+    @_wrapped_image_collection
+    def build(self, path: str, tag: str, rm: bool = True, **kwargs) -> tuple[Image, Iterable[dict]]:
+        """
+        Build a Docker image from a directory containing the Dockerfile.
+
+        :return: A tuple containing the image object and the build logs.
+        """
+        image_object, image_logs = self.client.images.build(path=path, tag=tag, rm=rm, **kwargs)
+
+        return image_object, image_logs
+
     def find_host_network(self) -> Optional[str]:
         """
         Try to find the docker host network.
@@ -89,8 +130,18 @@ class DockerClient:
         """
         # If we're docker in docker running on a custom network, we need to inherit the
         # network settings, so we can access the resulting container.
+
+        # first to try to find the network the container runs in, if we can determine
+        container_id = utils.get_running_in_container_id()
+        if container_id:
+            with contextlib.suppress(Exception):
+                return self.network_name(container_id)
+
+        # if this results nothing, try to determine the network based on the
+        # docker_host
         try:
-            docker_host = ipaddress.IPv4Address(self.host())
+            host_ip = socket.gethostbyname(self.host())
+            docker_host = ipaddress.IPv4Address(host_ip)
             # See if we can find the host on our networks
             for network in self.client.networks.list(filters={"type": "custom"}):
                 if "IPAM" in network.attrs:
@@ -101,7 +152,7 @@ class DockerClient:
                             continue
                         if docker_host in subnet:
                             return network.name
-        except ipaddress.AddressValueError:
+        except (ipaddress.AddressValueError, OSError):
             pass
         return None
 
@@ -149,49 +200,65 @@ class DockerClient:
         network_name = self.network_name(container_id)
         return container["NetworkSettings"]["Networks"][network_name]["Gateway"]
 
+    def get_connection_mode(self) -> ConnectionMode:
+        """
+        Determine the connection mode.
+
+        See https://github.com/testcontainers/testcontainers-python/issues/475#issuecomment-2407250970
+        """
+        if c.connection_mode_override:
+            return c.connection_mode_override
+        localhosts = {"localhost", "127.0.0.1", "::1"}
+        if not utils.inside_container() or self.host() not in localhosts:
+            # if running not inside a container or with a non-local docker client,
+            # connect ot the docker host per default
+            return ConnectionMode.docker_host
+        elif self.find_host_network():
+            # a host network could be determined, indicator for DooD,
+            # so we should connect to the bridge_ip as the container we run in
+            # and the one we started are connected to the same network
+            # that might have no access to either docker_host or the gateway
+            return ConnectionMode.bridge_ip
+        # default for DinD
+        return ConnectionMode.gateway_ip
+
     def host(self) -> str:
         """
         Get the hostname or ip address of the docker host.
         """
-        # https://github.com/testcontainers/testcontainers-go/blob/dd76d1e39c654433a3d80429690d07abcec04424/docker.go#L644
-        # if os env TC_HOST is set, use it
-        host = os.environ.get("TC_HOST")
+        host = c.tc_host_override
         if host:
             return host
         try:
             url = urllib.parse.urlparse(self.client.api.base_url)
-
         except ValueError:
-            return None
-        if "http" in url.scheme or "tcp" in url.scheme:
+            return "localhost"
+        if "http" in url.scheme or "tcp" in url.scheme and url.hostname:
+            # see https://github.com/testcontainers/testcontainers-python/issues/415
+            if url.hostname == "localnpipe" and utils.is_windows():
+                return "localhost"
             return url.hostname
-        if inside_container() and ("unix" in url.scheme or "npipe" in url.scheme):
-            ip_address = default_gateway_ip()
+        if utils.inside_container() and ("unix" in url.scheme or "npipe" in url.scheme):
+            ip_address = utils.default_gateway_ip()
             if ip_address:
                 return ip_address
         return "localhost"
 
+    def login(self, auth_config: DockerAuthInfo) -> None:
+        """
+        Login to a docker registry using the given auth config.
+        """
+        login_info = self.client.login(**auth_config._asdict())
+        LOGGER.debug(f"logged in using {login_info}")
 
-@ft.cache
-def read_tc_properties() -> dict[str, str]:
-    """
-    Read the .testcontainers.properties for settings. (see the Java implementation for details)
-    Currently we only support the ~/.testcontainers.properties but may extend to per-project variables later.
-
-    :return: the merged properties from the sources.
-    """
-    tc_files = [item for item in [TC_GLOBAL] if exists(item)]
-    if not tc_files:
-        return {}
-    settings = {}
-
-    for file in tc_files:
-        tuples = []
-        with open(file) as contents:
-            tuples = [line.split("=") for line in contents.readlines() if "=" in line]
-            settings = {**settings, **{item[0].strip(): item[1].strip() for item in tuples}}
-    return settings
+    def client_networks_create(self, name: str, param: dict):
+        labels = create_labels("", param.get("labels"))
+        return self.client.networks.create(name, **{**param, "labels": labels})
 
 
 def get_docker_host() -> Optional[str]:
-    return read_tc_properties().get("tc.host") or os.getenv("DOCKER_HOST")
+    return c.tc_properties_get_tc_host() or os.getenv("DOCKER_HOST")
+
+
+def get_docker_auth_config() -> Optional[str]:
+    return c.docker_auth_config
