@@ -5,7 +5,7 @@ ArangoDB container support.
 import typing
 from os import environ
 
-from testcontainers.core.config import TIMEOUT
+from testcontainers.core.config import testcontainers_config as c
 from testcontainers.core.generic import DbContainer
 from testcontainers.core.utils import raise_for_deprecated_parameter
 from testcontainers.core.waiting_utils import wait_for_logs
@@ -90,4 +90,4 @@ class ArangoDbContainer(DbContainer):
         return f"http://{self.get_container_host_ip()}:{port}"
 
     def _connect(self) -> None:
-        wait_for_logs(self, predicate="is ready for business", timeout=TIMEOUT)
+        wait_for_logs(self, predicate="is ready for business", timeout=c.timeout)
