@@ -23,6 +23,8 @@ _DEFAULT_DEV_COMMAND = "start-dev"
 
 
 class KeycloakContainer(DockerContainer):
+    has_realm_imports = False
+
     """
     Keycloak container.
 
@@ -32,7 +34,7 @@ class KeycloakContainer(DockerContainer):
 
             >>> from testcontainers.keycloak import KeycloakContainer
 
-            >>> with KeycloakContainer(f"quay.io/keycloak/keycloak:24.0.1") as keycloak:
+            >>> with KeycloakContainer(f"quay.io/keycloak/keycloak:25.0.4") as keycloak:
             ...     keycloak.get_client().users_count()
             1
     """
@@ -43,12 +45,16 @@ class KeycloakContainer(DockerContainer):
         username: Optional[str] = None,
         password: Optional[str] = None,
         port: int = 8080,
+        management_port: int = 9000,
+        cmd: Optional[str] = _DEFAULT_DEV_COMMAND,
     ) -> None:
         super().__init__(image=image)
         self.username = username or os.environ.get("KEYCLOAK_ADMIN", "test")
         self.password = password or os.environ.get("KEYCLOAK_ADMIN_PASSWORD", "test")
         self.port = port
-        self.with_exposed_ports(self.port)
+        self.management_port = management_port
+        self.with_exposed_ports(self.port, self.management_port)
+        self.cmd = cmd
 
     def _configure(self) -> None:
         self.with_env("KEYCLOAK_ADMIN", self.username)
@@ -56,29 +62,56 @@ class KeycloakContainer(DockerContainer):
         # Enable health checks
         # see: https://www.keycloak.org/server/health#_relevant_options
         self.with_env("KC_HEALTH_ENABLED", "true")
-        # Starting Keycloak in development mode
+        # Start Keycloak in development mode
         # see: https://www.keycloak.org/server/configuration#_starting_keycloak_in_development_mode
-        self.with_command(_DEFAULT_DEV_COMMAND)
+        if self.has_realm_imports:
+            self.cmd += " --import-realm"
+        self.with_command(self.cmd)
 
     def get_url(self) -> str:
         host = self.get_container_host_ip()
         port = self.get_exposed_port(self.port)
         return f"http://{host}:{port}"
 
+    def get_management_url(self) -> str:
+        host = self.get_container_host_ip()
+        port = self.get_exposed_port(self.management_port)
+        return f"http://{host}:{port}"
+
     @wait_container_is_ready(requests.exceptions.ConnectionError, requests.exceptions.ReadTimeout)
     def _readiness_probe(self) -> None:
-        # Keycloak provides an REST API endpoints for health checks: https://www.keycloak.org/server/health
-        response = requests.get(f"{self.get_url()}/health/ready", timeout=1)
+        # Keycloak provides REST API endpoints for health checks: https://www.keycloak.org/server/health
+        try:
+            # Try the new health endpoint for keycloak 25.0.0 and above
+            # See https://www.keycloak.org/docs/25.0.0/release_notes/#management-port-for-metrics-and-health-endpoints
+            response = requests.get(f"{self.get_management_url()}/health/ready", timeout=1)
+        except requests.exceptions.ConnectionError:
+            response = requests.get(f"{self.get_url()}/health/ready", timeout=1)
         response.raise_for_status()
-        if self._command == _DEFAULT_DEV_COMMAND:
+        if _DEFAULT_DEV_COMMAND in self._command:
             wait_for_logs(self, "Added user .* to realm .*")
 
     def start(self) -> "KeycloakContainer":
-        self._configure()
         super().start()
         self._readiness_probe()
         return self
 
+    def with_realm_import_file(self, realm_import_file: str) -> "KeycloakContainer":
+        file = os.path.abspath(realm_import_file)
+        if not os.path.exists(file):
+            raise FileNotFoundError(f"Realm file {file} does not exist")
+        self.with_volume_mapping(file, "/opt/keycloak/data/import/realm.json")
+        self.has_realm_imports = True
+        return self
+
+    def with_realm_import_folder(self, realm_import_folder: str) -> "KeycloakContainer":
+        folder = os.path.abspath(realm_import_folder)
+        if not os.path.exists(folder):
+            raise FileNotFoundError(f"Realm folder {folder} does not exist")
+        self.with_volume_mapping(folder, "/opt/keycloak/data/import/")
+        self.has_realm_imports = True
+        return self
+
     def get_client(self, **kwargs) -> KeycloakAdmin:
         default_kwargs = {
             "server_url": self.get_url(),
